use super::ActionRunner;

pub(crate) struct DeleteContentBackward;

pub(crate) struct DeleteContentForward;

fn delete(start: usize, end: usize, mut doc: Vec<u16>) -> Vec<u16> {
    if start > doc.len() + 1 {
        return doc;
    }

    let mut start_idx = start;
    let mut end_idx = end;
    unsafe {
        while start_idx > 0 && matches!(doc.get_unchecked(start_idx), 0xDC00..=0xDFFF) {
            start_idx -= 1;
        }

        let end_byte = doc.get_unchecked(end_idx - 1);
        if end_idx < doc.len() && end_byte > &0xD800 && end_byte < &0xDC00 {
            end_idx += 1;
        }
    }

    doc.drain(start_idx..end_idx);
    doc
}

impl ActionRunner for DeleteContentBackward {
    fn run(&self, start: usize, end: usize, _data: Vec<u16>, doc: Vec<u16>) -> Vec<u16> {
        if (start == 0 && start == end ) || end > doc.len() { return doc; }
        let (end, start) = if start == end { (start, start - 1) } else { (end, start) };

        delete(start, end, doc)
    }
}

impl ActionRunner for DeleteContentForward {
    fn run(&self, start: usize, end: usize, _data: Vec<u16>, doc: Vec<u16>) -> Vec<u16> {
        if start >= doc.len() { return doc; }

        let end = if start == end { start + 1 } else { end };
        delete(start, end, doc)
    }
}
